/*

DESCRIPTION:    Test firmware for driving half-bridge isolation bias 
                transformer.

SYNTAX:         n/a

KNOWN BUGS:     n/a

COPYRIGHT:      (c) 2022, Brian Cornell, all rights reserved.

DEPENDENCIES:   see include file listing.

 TARGET MCU:    PIC12F1571

HW REVISION:    220720

VERSION:        0.1

MODIFICATION HISTORY:
 
    25 Jul 22, BWC, initial development.
    

*/


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * COMPILER & CLIB INCLUDES.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <pic.h>
#include <xc.h>

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * FIRMWARE INCLUDES.
 * + binary.h:          Boolean constants shared across firmware projects.
 * + config.h:          PIC config registers
 * + _xtal_freq.h:      sets the constant _XTAL_FREQ to the system oscillator
 *                      frequency (Fosc).  Used by the __delayxxx macros.
 * + isob.h:            main include file for this project.
 * 
 * PROJECT FOLDER ORGANIZATION & LOCATION OF INCLUDES.
 * /pvbc-210218-fw-v3_0 charger (PVBC) project
 * /pvb-common.X        includes shared by charger (PVBC) & balancer (PVBB).
 * /microchip/code      includes for use across multiple projects.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#include <binary.h>                     //folder microchip/code
#include "config.h"
#ifndef _XTAL_FREQ
#include "_xtal_freq.h"
#endif
#include "isob.h"


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * MODULE DATA.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


/*Function main.  Control program entry point (from root code) for control 
 * program. All aspects of operation begin & end here. At entry the 
 * Initialization function is called to configure peripherals, data objects, 
 * etc.
 * 
 * The WDT (Watchdog) timer is enabled for hard fault recovery and the RUN loop 
 * entered: it is an infinite loop that consists of several discrete blocks that
 * manage all aspects of operation. They are ordered by dependencies (e.g. data
 * collection must happen before fault detection) and/or criticality (e.g. fault
 * shutdown before console service). Refer to the documentation within each 
 * block for details.
 * 
 * Returns:     never
 *
 * Parameters:  none
 *
*/
void main(void)
{
    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * FUNCTION DATA.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * INITIALIZATION.
     * 
     * Function Initialize configures the MCU and peripherals.  Afterwards a 
     * delay provides settle time.  Local objects are initialized before
     * entering the RUN loop:
     +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    Initialize();
    __delay_ms(S_TM_INT);


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * RUN LOOP PRIORITIZATION & WATCHDOG MANAGEMENT.
     * 
     * Once entered, the run-time loop's only exit points are a MCLR reset or 
     * power cycle.
     * 
     * The CWG is enabled, and it is expected that Initialization set the PWM
     * duty cycle to S_DC_MIN. The DC is incremented at S_TM_DCDELAY time
     * intervals for a soft start. Once at S_DC_RUN no further action is taken
     * and the MCU sits in an infinite loop.
     * 
     * The watchdog timer is expected to be on in the run loop regardless of
     * state.  It should be set to a value that accounts for the longest loop
     * time with at least a 100% margin.  If it must be disabled in anticipation
     * of a long delay then it is the responsibility of that code block to 
     * re-enable.
     +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    S_AL_WDT_ENABLE=TRUE;                   //Enable watchdog

    while (TRUE)                            //Begin RUN loop
    {
        CLRWDT();                           //Clear watchdog timer
        if (RA2) G1ASE=FALSE;               //Check enable pin to start/stop
        else G1ASE=TRUE;
    }
} //End function main


/*Function Initialize. Called immediately after boot by function main:
 *  + Sets system clock speed
 *  + Configures pins, peripherals, interrupts
 *
 * Returns:     none
 *
 * Parameters:  none
 *
*/
void Initialize(void)
{
    di();                               //No interrupts during initialization    

    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * INITIALIZE SYSTEM CLOCK & WATCHDOG.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    OSCCON=CFG_OSC_C;                   //Configure system clock
    while (!(HFIOFL));                  //Wait for oscillator to stabilize
    WDTCON = CFG_WDT_CON;               //Watchdog


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * SET PORT CONFIGURATION:
     * + Enable weak pull-ups
     * + Set digital latch initial states
     * + Pin in/out-put states
     * + Analog / digital
     * + Weak pull-ups
     * + Open drain (tri-state)
     * + Slew rate
     * + Logic input levels
     * + Peripheral pin selection (mapping)
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    nWPUEN=FALSE;
    PORTA=S_IN_PORTA;
    TRISA=IO_PORTA;
    ANSELA=AD_PORTA;
    WPUA=WP_PORTA;
    ODCONA=OD_PORTA;
    SLRCONA=SR_PORTA;
    INLVLA=IL_PORTA;
    APFCON=IO_APFCON;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * CONFIGURE PWMs:
     * 1=CWG
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    PWM1INTE=CFG_PWM_1_INTE;
    PWM1CLKCON=CFG_PWM_1_CLK;
    PWM1LDCON=CFG_PWM_1_LDCON;
    PWM1OFCON=CFG_PWM_1_OFCON;
    PWM1PH=CFG_PWM_1_PHASE;
    PWM1DC=CFG_PWM_1_DC;
    PWM1PR=CFG_PWM_1_PERIOD;
    PWM1OF=CFG_PWM_1_OFFSET;
    PWM1TMR=CFG_PWM_1_TIMER;
    PWM1CON=CFG_PWM_1_CON;
    

    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * CONFIGURE CWG:
     * Half-bridge / push-pull drive. Set CON0 last since it's expected that the
     * Enable bit is set but don't want it to start. The ASE bit should also be
     * set to ensure that the CWG does not start when enabled.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
    CWG1CON1=CFG_CWG_1_CON1;
    CWG1CON2=CFG_CWG_1_CON2;
    CWG1DBR=CFG_CWG_1_DBR;
    CWG1DBF=CFG_CWG_1_DBF;
    CWG1CON0=CFG_CWG_1_CON0;


    /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     * INITIALIZATION COMPLETE.
     *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

    return;
} //End function Initialize



//End of file
